#!/usr/bin/perl -w
# Author        : Pedro Velho
# last modified : 05/11/2008


$workingdir="/home/velho/Development/workspace/simgrid/examples/msg/gtnets";
if($workingdir eq ""){
    print "Please edit me with a correct work directory, use absolut path is mandatory!\n";
    exit 1;
}
$thisdir= `pwd`;
chomp $thisdir;

# ****** 1- Parameter Message Size (S) ******
# 10^5 KBytes
# param1: message size (S) (IN BYTES)
@message_size_list=("100000000");

# ****** 2- Parameter Latency (L) ****** 
@latency_list=( "0.000", "0.001", "0.010", "0.020", "0.050", "0.004", "0.008", "0.030", "0.040", "0.060", "0.070", "0.080",  "0.090", "0.100", "0.110", "0.120", "0.130", "0.140", "0.150", "0.160", "0.170", "0.180", "0.190", "0.200");


# ****** 3- Bandwidth (B) ******
@bandwidth_list=( "1.000000e+04", "1.000000e+05", "1.000000e+06", "1.000000e+07", "1.000000e+08",  "1.000000e+09", "4.641598e+08", "2.154440e+09", "2.154435e+04", "4.641600e+09",  "4.641590e+04", "2.154436e+05", "4.641592e+05", "2.154437e+06", "4.641594e+06",  "2.154438e+07", "4.641596e+07", "2.154439e+08", "2.000000e+04", "3.000000e+04", "4.000000e+04", "5.000000e+04", "6.000000e+04", "7.000000e+04", "8.000000e+04", "9.000000e+04", "2.000000e+05", "3.000000e+05", "4.000000e+05", "5.000000e+05", "6.000000e+05", "7.000000e+05", "8.000000e+05", "9.000000e+05", "2.000000e+06", "3.000000e+06", "4.000000e+06", "5.000000e+06", "6.000000e+06", "7.000000e+06", "8.000000e+06", "9.000000e+06");

# ****** 4- Model (M) ******
@model_list=("GTNets","CM02","LV08");

$task_begin=0;
$task_end=0;

#########################################
# Print error and exit
#########################################
sub printError {
    my($errorMsg) = $_[0];
    print "Error : ".$errorMsg."\n";
    print "Parse usage:\n";
    print "\tsweap_analyze parse\n\n";
    print "Sweep usage:\n";
    print "\tsweap_analyze sweep <task_begin> <task_end>\n\n";
    print "Examples\n";
    print "\tsweap_analyze 1 1 # executes task 1 (fix bandwidth[0] parameter)\n";
    print "\tsweap_analyze 1 3 # executes task from 1 up to 3\n\n";
    print "\nIn this program you have ".($#bandwidth_list+1)." tasks to parallelize.\n\n";
    exit 1;
}

if(scalar(@ARGV) < 1){ 
    printError("Needs at least one parameter sweep or parse");
}

if($ARGV[0] ne "sweep" && $ARGV[0] ne "parse"){
    printError("First argument should be sweep or parse");
}

if($ARGV[0] eq "sweep"){
    if(scalar(@ARGV) != 3){
	printError("Option sweep must be followed by two integers in task range.");
    }else {
	executeApplication();
    }
}

if($ARGV[0] eq "parse"){
    parseResults();
}

#########################################
# Parse and verify arguments range
#########################################
sub parseParameters() {

    #adjust to perl parameters (arrays are from 0 up size-1)
    $task_begin=$ARGV[1]-1;
    $task_end=$ARGV[2]-1;
    if($task_begin < 0){
	print "Starting from a not valuable array index aborting...\n";
	print "Check the correctness of first parameter.\n";
	exit 1;
    }
    print "Bandiwthd array size is : ",scalar(@bandwidth_list),"\n";
    if($task_end >= scalar(@bandwidth_list)){
	print "Error doing more tasks than we have to do aborting...\n";
	print "Check the correctness of second parameter.\n";
	exit 1;
    }
    if($task_begin > $task_end){
	print "Error starting point is greater then end point...\n";
	print "Second parameter must be greater.\n";
	exit 1;
    }
}



#########################################
# Execute application only once
#########################################
sub executeApplication {
    parseParameters();
    
    print "Changing working directory to ".$workingdir."\n";
    chdir($workingdir);

    $tmpoutputxmlpla = $thisdir."/tmp/plateforme-".($task_begin+1)."-".($task_end+1).".xml";
    $tmpoutputxmldep = $thisdir."/tmp/deployment-".($task_begin+1)."-".($task_end+1).".xml";
    $tmpoutputlog    = $thisdir."/tmp/tempotrace-".($task_begin+1)."-".($task_end+1).".log";

    open(LOGOUTPUT, ">" , $thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log") ||
	die "Sorry, couldn't open file ".$thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log for writting...\n";

    print LOGOUTPUT "\n";

    $it=0;
    for($it=$task_begin; $it <= $task_end; $it++) 
    {
	$bandwidth = $bandwidth_list[$it];
	print ">=============================================================<\n";
	print "========> Bandwidth (B) : ",$bandwidth," B/s (Bytes per second)\n";
	foreach $latency (@latency_list)
	{
	    print "========> Latency   (L) : ",$latency," s (seconds)\n";

	    foreach $size (@message_size_list)
	    {
		print "========> Size      (S) : ",$size," B (Bytes) \n";		
		foreach $model (@model_list)
		{
		    print "========> Model     (M) : ",$model,"\n";

		    # Log some stuff into the file to know used parameters
		    print LOGOUTPUT ">==================================================<\n";
		    print LOGOUTPUT "========> Bandwidth (B) : ",$bandwidth," B/s (Bytes per second)\n";
		    print LOGOUTPUT "========> Latency   (L) : ",$latency," s (seconds)\n";
		    print LOGOUTPUT "========> Size      (S) : ",$size," B (Bytes) \n";
		    print LOGOUTPUT "========> Model     (M) : ",$model,"\n";
		    
		
		    # Set parameters
		    system ("sed -e s/bw/".$bandwidth."/g -e s/lt/".$latency."/g ".$thisdir."/dumbbell-p-template.xml > ".$tmpoutputxmlpla);
		    system ("sed -e s/size/".$size."/g ".$thisdir."/dumbbell-d-template.xml > ".$tmpoutputxmldep);


		    # Run with parameters
		    print("Running command:\n".$workingdir."/gtnets ".$tmpoutputxmlpla." ".$tmpoutputxmldep." --cfg=workstation_model:compound --cfg=cpu_model:Cas01 --cfg=network_model:".$model." 2>&1\n");
		    
		    # Open temporary output file
		    open(TMPOUTPUT, $workingdir."/gtnets ".$tmpoutputxmlpla." ".$tmpoutputxmldep." --cfg=workstation_model:compound --cfg=cpu_model:Cas01 --cfg=network_model:".$model." 2>&1 | ") || die "Sorry, could't run application...\n";

		    
		    # Copy execution trace into global trace file
		    while(<TMPOUTPUT>)
		    {
			my $line = $_;
			chomp $line;
			print LOGOUTPUT $line."\n";
		    }
		    print LOGOUTPUT "=========================><=========================\n";
		}
	    }
	}
	print "==============================><=============================\n";
    }
    
    close(LOGOUTPUT);
}

#########################################
# Parse resulting values
# Genarate input file for R
#########################################
sub parseResults {

    open(RAWOUTPUT, ">".$thisdir."/dat/raw.data") ||
	die "Sorry, couldn't open file ".$thisdir."/dat/raw.data for writting...\n";

    print RAWOUTPUT "Bandwidth Latency Size Model ResultBandwidthFlowA ResultBandwidthFlowB RemainingA RemainingB\n";

    my($formater);
    my ($flowA, $flowB, $flowsparsed, $remaingA, $remaingB);
    my($B, $L ,$S, $M);  
    my($counter);
    $counter=1;
    
    for($i=1; $i <= scalar(@bandwidth_list); $i++) 
    {	
	open(LOGOUTPUT, "<".$thisdir."/log/trace-file-".$i."-".$i.".log") ||
	    die "Sorry, I coundn't open file ".$thisdir."/log/trace-file-".$i."-".$i.".log for reading...";
	
	$formater="";
	
	$flowA = $flowB = 0;
	$flowsparsed = 0;
	
	# Copy execution trace into global trace file
	while(<LOGOUTPUT>)
	{ 

	    my $line = $_; chomp $line;
	  
	    if($flowsparsed == 2){ 
		$formater= "$formater $flowA $flowB $remaingA $remaingB"; print RAWOUTPUT $counter++," ",$formater,"\n"; 
		$formater="";
		$flowsparsed = 0;
		$remaingA = 0; 
		$remaingB = 0;
		$flowA = $flowB = 0; 
	    }

	    # Find first finishing flow 
	    if($line =~ /FLOW.1./){ 
		my($flow,$remain)=split(/;/, $line);
		$flowA = get_value($flow);
		$remaingA = get_value($remain);
		$flowsparsed++; 
		next; 
	    }
    
	    # Find last finishing flow 
	    if($line =~ /FLOW.2./){ 
		my($flow,$remain)=split(/;/, $line);
		$flowB = get_value($flow);
		$remaingB = get_value($remain);
		$flowsparsed++; 
		next; 
	    }

	    # Find bandwidth parameter 
	    if($line =~ /Bandwidth/){ 
		$B = get_value($line); 
		$formater = $formater." ".$B; 
		next; 
	    }

	    # Find latency parameter 
	    if($line =~ /Latency/){ 
		$L = get_value($line); $formater = $formater." ".$L;
		next;
	    }

	    # Find size parameter 
	    if($line =~ /Size/){ 
		$S = get_value($line);
		$formater = $formater." ".$S;
		next;
	    }

	    # Find model parameter 
	    if($line =~ /Model/){ 
		$line =~ s/^.*\s+Model\s+\(M\)\s+:\s+(.*)\s*/$1/x; 
		$M = $line;
		$formater = $formater." ".$M; 
		next; 
	    }

	}
	close(LOGOUTPUT);
    }
    close(RAWOUTPUT);
}




#########################################
# Parse numerical value within a line
#########################################
sub get_value {
    my($line)=shift;
    $line=" $line";

    $line =~ s/^
                 .*[^\d\.+-eE]+         # anything except something in a number
                 (
                   [+-]?             # first, match an optional sign
                   (                 # then match integers or f.p. mantissas:
                       \d+            # start out with a ...
                       (
                           \.\d*      # mantissa of the form a.b or a.
                       )?             # ? takes care of integers of the form a
                       |\.\d+         # mantissa of the form .b
                     )
                     ([eE][+-]?\d+)?  # finally, optionally match an exponent
                   )
		 \D*$/$1/x;
    return $line;
}
