

# Create a .data file for paltform and experiment uses a refmodel as bases to compute the log error.
# @param
# @platform A file name in levels(dataTable$Model)
# @exp      An experience number must contain in unique(dataTable$Experiment)
# @refModel A string containing a reference model in levels(dataTable$Model)
# @orderBy  An array of columns to sort the frame
createDataInput = function(dataTable, platform='waxman_50_1.xml',exp=1,refModel='GTNets',orderBy=c(3), returnStats=FALSE){

                                        # Select for a specific plaformFile and experience
  myNewFrame = dataTable[((dataTable$File == platform)&(dataTable$Experiment == exp)),] 

                                        # select the bandwidth share for referential model
  REFMODEL = myNewFrame[(myNewFrame$Model == refModel),]$BandwidthShare

                                        # open ouputfile
  outputFilename = paste("./tmp/",platform,"-",exp,".data",sep="")
  outFile = file(outputFilename, "w")

                                        # write header line
  cat(file=outFile, "#",levels(myNewFrame$Model),"and repective errors\n", sep=" ")
  
                                    # initialize print frame with 2 columns
  printFrame = data.frame(seq(1,length(REFMODEL),by=1))
  colindex=2


  ############# DIRTY HACK
  VECMAX = c()
  VECMEAN = c()
  VECSD = c()
  #############
  
  for(model in levels(myNewFrame$Model)){
      BWSMODEL = myNewFrame[(myNewFrame$Model == model),]$BandwidthShare
      ERROR = log(BWSMODEL) - log(REFMODEL)
      totalERROR = sqrt(sum(ERROR^2)/length(ERROR))
      cat(file=outFile, "# TOTAL error for ",model," : ",totalERROR,"\n",sep="")
      cat(file=outFile, "# MEAN  error for ",model," : ",mean(abs(ERROR)),"\n",sep="")
      cat(file=outFile, "# MAX   error for ",model," : ",max(abs(ERROR)),"\n",sep="")
      cat(file=outFile, "# SD    error for ",model," : ",sd(abs(ERROR)),"\n",sep="")

      ############# DIRTY HACK
      if(refModel != model){
        VECMAX = c(VECMAX, max(abs(ERROR)))
        VECMEAN = c(VECMEAN, mean(abs(ERROR)))
        VECSD = c(VECSD, sd(abs(ERROR)))
      }
      #############
      
      printFrame[,colindex] = BWSMODEL
      colindex = colindex + 1
      printFrame[,colindex] = ERROR
      colindex = colindex + 1
  }


                                        #printFrame = data.frame(PRINTARRAY[,1],PRINTARRAY[,2])
                                        # sort data frame, default value is first error
  printFrame = printFrame[order(printFrame[,orderBy]),]

                                        # reinitialize index column
  printFrame[,1] = seq(1,length(REFMODEL),by=1)
  printFrame = data.frame(printFrame)
  write.table(file=outFile, row.names=FALSE, col.names=FALSE, printFrame)
  close(outFile)


  if(returnStats){
    return (c(VECMAX,VECMEAN,VECSD))
  } else {
    return (outputFilename)
  }
}



plotGnuplot = function(dataTable, minError = -2.5, maxError = 2.5, ticsError=0.5, minBw = 0, maxBw = 200, ticsBw=20, platform="WAN.xml", exp=1,orderBy=3){

  dataFileName = createDataInput(platform=platform, exp=exp, dataTable=dataTable, orderBy=orderBy)
  outFile = file("tmp/gnuplot.script", "w")

  cat(file=outFile, "unset key  \n",sep="")
  cat(file=outFile, "set xrange [0:151]\n",sep="")
  cat(file=outFile, "set xtics 20\n",sep="")
  cat(file=outFile, "set lmargin 10\n",sep="")
  cat(file=outFile, "set rmargin 2\n",sep="")
  cat(file=outFile, "set multiplot\n",sep="")

  cat(file=outFile, "set xlabel \"Flow ID\" \n",sep="")
  cat(file=outFile, "set ylabel \"Error\" \n",sep="")
  cat(file=outFile, "set size 1,0.4\n",sep="")
  cat(file=outFile, "set origin 0.0,0.0\n",sep="")
  cat(file=outFile, "set yrange [",minError,":",maxError,"]\n",sep="")
  cat(file=outFile, "set ytics ",ticsError,"\n",sep="")
  cat(file=outFile, "set tmargin 0\n",sep="")
  cat(file=outFile, "plot 0, '",dataFileName,"' u 1:3 w points, '",dataFileName,"' u 1:7 w points\n",sep="")

  cat(file=outFile, "set size 1,0.6\n",sep="")
  cat(file=outFile, "set key box\n",sep="")
  cat(file=outFile, "set ylabel \"Bandwidth (KBps)\" \n",sep="")
  cat(file=outFile, "set yrange [",minBw,":",maxBw,"]\n",sep="")
  cat(file=outFile, "set ytics ",ticsBw,"\n",sep="")
  cat(file=outFile, "unset xtics\n",sep="")
  cat(file=outFile, "set origin 0.0,0.4\n",sep="")
  cat(file=outFile, "set format x \"\"\n",sep="")
  cat(file=outFile, "set xlabel \"\"\n",sep="")  
  cat(file=outFile, "RATIO=1000\n\n",sep="")
  cat(file=outFile, "plot '",dataFileName,"' u 1:($4/RATIO) with points t \"GTNets\", '",dataFileName,"' u 1:($2/RATIO) w points t \"CM02\", '",dataFileName,"' u 1:($6/RATIO) w points t \"LegrandVelho\"\n\n",sep="")

  cat(file=outFile, "set nomultiplot\n",sep="")
  close(outFile)

  system("gnuplot -persist tmp/gnuplot.script")
}



resultTable = read.table("dat/raw.data")
plotGnuplot(resultTable)



Platform = levels(resultTable$File)
Exp = seq(0, length=length(length(Platform)), by=1)
MaxCM02 = seq(0, length=length(length(Platform)), by=1)
MaxLV = seq(0, length=length(length(Platform)), by=1)
MeanCM02 = seq(0, length=length(length(Platform)), by=1)
MeanLV = seq(0, length=length(length(Platform)), by=1)
SdCM02 = seq(0, length=length(length(Platform)), by=1)
SdLV = seq(0, length=length(length(Platform)), by=1)
globalResultFrame = data.frame(Platform,Exp,MaxCM02,MaxLV, MeanCM02, MeanLV,SdCM02, SdLV)
glFrameIndex = 1

for(platformFile in levels(resultTable$File)){
  for(experience in sort(unique(resultTable$Experiment)) ){
############# DIRTY HACK
    errors = createDataInput(resultTable, platform=platformFile, exp=experience, returnStats=TRUE)
    globalResultFrame[glFrameIndex,] = c(platformFile,experience,errors[1],errors[2],errors[3],errors[4],errors[5],errors[6])
    glFrameIndex = glFrameIndex + 1
#############
  }
}


prMaxCM02 = c()
prMaxLV = c()
prMeanCM02 = c()
prMeanLV = c()
prSdCM02 = c()
prSdLV = c()
expe = c()
platform = c()
for(plat in levels(globalResultFrame$Platform)){
  tmpFrame = globalResultFrame[((globalResultFrame$MaxLV != Inf)&(globalResultFrame$Platform == plat)),]
  tmpFrame = tmpFrame[(tmpFrame$MaxLV == max(tmpFrame$MaxLV)),]
  prMaxCM02 = c(  prMaxCM02 , as.numeric(tmpFrame$MaxCM02))
  prMaxLV = c(  prMaxLV , as.numeric(tmpFrame$MaxLV))
  prMeanCM02 = c(  prMeanCM02 , tmpFrame$MeanCM02)
  prMeanLV = c(  prMeanLV , tmpFrame$MeanLV)
  prSdCM02 = c(  prSdCM02 , tmpFrame$SdCM02)
  prSdLV = c(  prSdLV , tmpFrame$SdLV)
  expe = c(  expe , tmpFrame$Exp)
  platform = c( platform, plat)
}
prMaxCM02 = formatC(as.numeric(prMaxCM02), digits=4, format='f')
prMeanCM02 = formatC(as.numeric(prMeanCM02), digits=4, format='f')
prSdCM02 = formatC(as.numeric(prSdCM02), digits=4, format='f')
prMaxLV = formatC(as.numeric(prMaxLV), digits=4, format='f')
prMeanLV = formatC(as.numeric(prMeanLV), digits=4, format='f')
prSdLV = formatC(as.numeric(prSdLV), digits=4, format='f')
frameTable = data.frame(platform,expe,prMaxCM02,prMeanCM02,prSdCM02,prMaxLV,prMeanLV,prSdLV)
frameTable = frameTable[order(frameTable[,3]),]
write.table(file= "", quote=FALSE, row.names=FALSE, col.names=FALSE, frameTable, sep=" & ")



tmpGnuplot2D = function (X, title="no name",  orderBy=1, minY=0, maxY=2){
  inputFile = file("tmp/gnuplot2d.data", "w")
  cat(file=inputFile, sort(X), sep="\n")
  close(inputFile)
  
  outFile = file("tmp/gnuplot2d.script", "w")
  cat(file=outFile, "set yrange [",minY,":",maxY,"]\n", sep="")
  cat(file=outFile, "set title \"",title,"\"\n", sep="")
  cat(file=outFile, "plot 0, 'tmp/gnuplot2d.data'\n", sep="")
  close(outFile)
  system("gnuplot -persist tmp/gnuplot2d.script") 
}



plotGnuplotWithRatio = function(A, B, minRatio = -2.5, maxRatio = 2.5, ticsRatio=0.5, minX=0, maxX=151, ticsX=20, minValue = 0, maxValue = 1, ticsValue=0.1, orderBy=1, titleA="A", titleB="B", xTitle="ID", yTitle="Value", generateEps=FALSE){

  tmpFrame = data.frame(as.numeric(A), as.numeric(B), log(as.numeric(A)) - log(as.numeric(B)))
  tmpFrame = tmpFrame[order(tmpFrame[,orderBy]),]

  epsOutput = paste(titleA,"-",titleB,".eps",sep="")
  
  dataFileName = "tmp/gnuplotWithRatio.data"
  write.table(file=dataFileName, row.names=FALSE, col.names=FALSE, tmpFrame)
  outFile = file("tmp/gnuplotWithRatio.script", "w")

  if(generateEps == TRUE) {
    cat(file=outFile, "set term postscript eps enhanced color\n",sep="")
    cat(file=outFile, "set output \"",epsOutput,"\"\n",sep="")
  }
  
  cat(file=outFile, "unset key  \n",sep="")
  cat(file=outFile, "set xrange [",minX,":",maxX,"]\n",sep="")
  cat(file=outFile, "set xtics ",ticsX,"\n",sep="")
  cat(file=outFile, "set lmargin 10\n",sep="")
  cat(file=outFile, "set rmargin 2\n",sep="")
  cat(file=outFile, "set multiplot\n",sep="")
  cat(file=outFile, "set xlabel \"",xTitle,"\" \n",sep="")
  cat(file=outFile, "set ylabel \"Ratio\" \n",sep="")
  cat(file=outFile, "set size 1,0.4\n",sep="")
  cat(file=outFile, "set origin 0.0,0.0\n",sep="")
  cat(file=outFile, "set yrange [",minRatio,":",maxRatio,"]\n",sep="")
  cat(file=outFile, "set ytics ",ticsRatio,"\n",sep="")
  cat(file=outFile, "set tmargin 1\n",sep="")
  cat(file=outFile, "plot 0, '",dataFileName,"' u 3 w histograms\n",sep="")
  cat(file=outFile, "set size 1,0.6\n",sep="")
  cat(file=outFile, "set key box\n",sep="")
  cat(file=outFile, "set ylabel \"",yTitle,"\" \n",sep="")
  cat(file=outFile, "set yrange [",minValue,":",maxValue,"]\n",sep="")
  cat(file=outFile, "set ytics ",ticsValue,"\n",sep="")
  cat(file=outFile, "set grid\n",sep="")
  cat(file=outFile, "unset xtics\n",sep="")
  cat(file=outFile, "set origin 0.0,0.4\n",sep="")
  cat(file=outFile, "set format x \"\"\n",sep="")
  cat(file=outFile, "set xlabel \"\"\n",sep="")  
  cat(file=outFile, "plot '",dataFileName,"' u 1 w points t \"",titleA,"\", '",dataFileName,"' u 2 w points t \"",titleB,"\"\n",sep="")
  cat(file=outFile, "set nomultiplot\n",sep="")
  close(outFile)
  system("gnuplot -persist tmp/gnuplotWithRatio.script")
}


plotGnuplotWithRatio(as.numeric(globalResultFrame$MeanCM02), as.numeric(globalResultFrame$MeanLV), titleA="Maxmin", titleB="Improved", maxValue=1, ticsValue=0.5, minRatio=-3, maxRatio=3, ticsRatio=1,  minX=1, maxX=170, ticsX=10, orderBy=1, xTitle="Experiment", yTitle="Mean Error", generateEps=FALSE)




tmpGnuplot2D(title="Mean Ratio", log(as.numeric(globalResultFrame$MeanCM02)) - log(as.numeric(globalResultFrame$MeanLV)) , minY = -2, maxY=2)

tmpGnuplot2D(title="Ratio CM02", as.numeric(globalResultFrame$MeanCM02) , minY = 0, maxY=1)


