#!/usr/bin/perl -w
# Author        : Pedro Velho
# last modified : 10/12/2008
$workingdir="/home/velho/Development/projet-simgrid/simgrid/examples/msg/gtnets";
if($workingdir eq ""){
    print "Please edit me with a correct work directory, use absolut path is mandatory!\n";
    exit 1;
}
$thisdir= "/home/velho/Documents/Experiences/03-random/";


# ****** 1- Platform file list (P) ******
@platform_list=($thisdir."/platforms/brite/waxman_200_1H.xml",
		$thisdir."/platforms/brite/waxman_200_1.xml", 
		$thisdir."/platforms/brite/waxman_200_2H.xml", 
		$thisdir."/platforms/brite/waxman_200_2.xml", 
		$thisdir."/platforms/brite/waxman_200_3H.xml", 
		$thisdir."/platforms/brite/waxman_200_3.xml", 
		$thisdir."/platforms/brite/waxman_200_4H.xml", 
		$thisdir."/platforms/brite/waxman_200_4.xml", 
		$thisdir."/platforms/brite/waxman_50_1H.xml", 
		$thisdir."/platforms/brite/waxman_50_1.xml", 
		$thisdir."/platforms/brite/waxman_50_2H.xml", 
		$thisdir."/platforms/brite/waxman_50_2.xml", 
		$thisdir."/platforms/brite/waxman_50_3H.xml", 
		$thisdir."/platforms/brite/waxman_50_3.xml", 
		$thisdir."/platforms/brite/waxman_50_4H.xml", 
		$thisdir."/platforms/brite/waxman_50_4.xml");

#******* 2- Number of Flows (F)    *****
@flows_list = (10, 50, 100, 150, 200);

# ****** 3- Size (S) ******
# 1MB, 10MB, 100MB
@message_size_list = (1000000, 10000000, 100000000);

# ****** 4- Experiments (E) ******
$exp=10;

# ****** 5- Model (M) ******
@model_list=("CM02", "LegrandVelho", "Reno", "Reno2", "Vegas", "GTNets");

$task_begin=0;
$task_end=0;

#########################################
# Print error and exit
#########################################
sub printError {
    my($errorMsg) = $_[0];
    print "Error : ".$errorMsg."\n";
    print "Parse usage:\n";
    print "\tsweap_analyze parse\n\n";
    print "Sweep usage:\n";
    print "\tsweap_analyze sweep <task_begin> <task_end>\n\n";
    print "Examples\n";
    print "\tsweap_analyze 1 1 # executes task 1 (fix bandwidth[0] parameter)\n";
    print "\tsweap_analyze 1 3 # executes task from 1 up to 3\n\n";
    print "\nIn this program you have ".($#platform_list+1)." tasks to parallelize.\n\n";
    exit 1;
}

if(scalar(@ARGV) < 1){ 
    printError("Needs at least one parameter sweep or parse");
}

if($ARGV[0] ne "sweep" && $ARGV[0] ne "parse"){
    printError("First argument should be sweep or parse");
}

if($ARGV[0] eq "sweep"){
    if(scalar(@ARGV) != 3){
	printError("Option sweep must be followed by two integers in task range.");
    }else {
	executeApplication();
    }
}

if($ARGV[0] eq "parse"){
    parseResults();
}

#########################################
# Parse and verify arguments range
#########################################
sub parseParameters() {
    #adjust to perl parameters (arrays are from 0 up size-1)
    $task_begin=$ARGV[1]-1;
    $task_end=$ARGV[2]-1;
    if($task_begin < 0){
	print "Starting from a not valuable array index aborting...\n";
	print "Check the correctness of first parameter.\n";
	exit 1;
    }
    print "File list size is : ",scalar(@platform_list),"\n";
    if($task_end >= scalar(@platform_list)){
	print "Error doing more tasks than we have to do aborting...\n";
	print "Check the correctness of second parameter.\n";
	exit 1;
    }
    if($task_begin > $task_end){
	print "Error starting point is greater then end point...\n";
	print "Second parameter must be greater.\n";
	exit 1;
    }
}


#########################################
# Execute application only once
#########################################
sub executeApplication {
    parseParameters();
    
    print "Changing working directory to ".$workingdir."\n";
    chdir($workingdir);

    $tmpoutputlog    = $thisdir."/tmp/tempotrace-".($task_begin+1)."-".($task_end+1).".log";

    open(LOGOUTPUT, ">" , $thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log") ||
	die "Sorry, couldn't open file ".$thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log for writting...\n";

    $it=0;
    for($it=$task_begin; $it <= $task_end; $it++) 
    {
	$platform_file = $platform_list[$it];
    
	print LOGOUTPUT "\n";

	print ">=============================================================<\n";
	print "========> Platform File (P) : ",$platform_file,"\n";

	#find deployment file depending on platform file name
	$deployment_file = $platform_file;
	$deployment_file =~ s/.*\/(.*)\.xml$/$1/x;
	$deployment_file = $deployment_file."-d.xml";
	
#	print "))))) Deployment file: $deployment_file\n";


	foreach $nflows (@flows_list) {	
	    print "========> NFlows     (F) : ",$nflows,"\n";	    
	    
	    foreach $size (@message_size_list) {	    
		print "========> Size     (S) : ",$size,"\n";	    
		
		$it2=1;
		for($it2=1; $it2 <= $exp; $it2++){	    
		    print "========> Experiment     (E) : ",$it2,"\n";	    
   
		    $temp_dep_file = $deployment_file."-nflows-".$nflows."-size-".$size."-exp-".$it2;

		    #generate new deployment for each experiment if not exists
		    if( !( -r $thisdir."/deploy/".$temp_dep_file) ){
			chdir($thisdir);
			print $thisdir."/bin/generate_random_deployment.pl ".$platform_file." ".$nflows." ".$size."\n";
			system($thisdir."/bin/generate_random_deployment.pl ".$platform_file." ".$nflows." ".$size."\n");
			#save the experiment deployment file
			system("cp ".$thisdir."/deploy/".$deployment_file." ".$thisdir."/deploy/".$temp_dep_file);
			print("cp ".$thisdir."/deploy/".$deployment_file." ".$thisdir."/deploy/".$temp_dep_file."\n");
			chdir($workingdir);
		    }
		    
		    foreach $model (@model_list)
		    {
			print "========> Model     (M) : ",$model,"\n";		
			# Log some stuff into the file to know used parameters
			print LOGOUTPUT ">==================================================<\n";
			print LOGOUTPUT "========> Platform File (P) : ",$platform_file,"\n";
			print LOGOUTPUT "========> NFlows        (F) : ",$nflows,"\n";
			print LOGOUTPUT "========> Size          (S) : ",$size,"\n";
			print LOGOUTPUT "========> Experiment    (E) : ",$it2,"\n";
			print LOGOUTPUT "========> Model         (M) : ",$model,"\n";
			
			# Run with parameters
			print $workingdir."/gtnets ".$platform_file." ".$thisdir."/deploy/".$temp_dep_file." --cfg=workstation_model:compound --cfg=cpu_model:Cas01 --cfg=network_model:".$model."  2>/dev/null > ".$tmpoutputlog;
			system($workingdir."/gtnets ".$platform_file." ".$thisdir."/deploy/".$temp_dep_file." --cfg=workstation_model:compound --cfg=cpu_model:Cas01 --cfg=network_model:".$model."  2>/dev/null > ".$tmpoutputlog);
			
			# Open temporary output file
			open(TMPOUTPUT, "<".$tmpoutputlog) || 
			    die "Sorry, could't open temporary result file...\n";
			
			# Copy execution trace into global trace file
			while(<TMPOUTPUT>)
			{
			    my $line = $_;
			    chomp $line;
			    print LOGOUTPUT $line."\n";
			}
			print LOGOUTPUT "=========================><=========================\n";		
		    }
		}
	    }
	}
    }
    print "==============================><=============================\n";   
    close(LOGOUTPUT);
}



#########################################
# Parse resulting values
# Genarate input file for R
#########################################
sub parseResults {

    my($counter,$i);
    
    open(RAWOUTPUT, ">".$thisdir."/data/"."raw.data") || die "Sorry, couldn't open file ".$thisdir."data/raw.data for writting...\n";
    
    print RAWOUTPUT "File NFlows Size Experiment Model FlowID BandwidthShare CommRemaining\n";
    

    for($i=1; $i <= ($#platform_list+1); $i++) 
    {
	$tracefile = $thisdir."/log/"."trace-file-".($i)."-".($i).".log";

	open(LOGOUTPUT, "<", $tracefile) || die "Sorry, I coundn't open file ".$tracefile." for reading...";
	
	print $tracefile."\n";

	my($formater);
	my($File, $NFlow, $size, $Experiment, $Model, $FlowID, $BandwidthShare, $CommRemaining);
	# Copy execution trace into global trace file
	while(<LOGOUTPUT>)
	{
	    my $line = $_;
	    chomp $line;
	    
	    # Parse a flow
	    if($line =~ /FLOW/){
		$FlowID = $line;
		$FlowID =~ s/.*\[(\d+)\].*$/$1/x;
		my($BandwidthSh,$CommR)=split(/remaining/, $line);
		$BandwidthShare = get_value($BandwidthSh);
		$CommRemaining  = get_value($CommR);		
		print RAWOUTPUT $counter++." \"".$File."\" ".$NFlow." ".$size." ".$Experiment." \"".$Model."\" ".$FlowID." ".$BandwidthShare." ".$CommRemaining."\n";
		next;
	    }
	    
	    # Find a EXP label
	    if($line =~ /Experiment/){
		$Experiment = get_value($line);
		next;
	    }

	    # Find FILE
	    if($line =~ /Platform\s+File/){
		$File = $line;
		$File =~ s/^.*\(FILE\)\s*:\s*(.*)\s*$/$1/x;
		$File =~ s/^.*\/.*\/(.*)/$1/x;
		next;
	    }
	    
	    # Find model parameter
	    if($line =~ /Model/){
		$line =~ s/^.*\s+Model\s+\(M\)\s+:\s+(.*)\s*/$1/x;
		$Model = $line;
		next;
	    }

	    # Find Size
	    if($line =~ /Size/){
		$size = get_value($line);
		next;
	    }

	    # Find NFlow
	    if($line =~ /NFlow/){
		$NFlow = get_value($line);
		next;
	    }	    
	}
	close(LOGOUTPUT);
    }
    close(RAWOUTPUT);
}




#########################################
# Parse numerical value within a line
#########################################
sub get_value {
    my($line)=shift;
    $line=" $line";

    $line =~ s/^
                 .*[^\d\.+-eE]+       # anything except something in a number
                 (
                   [+-]?              # first, match an optional sign
                   (                  # then match integers or f.p. mantissas:
                       \d+            # start out with a ...
                       (
                           \.\d*      # mantissa of the form a.b or a.
                       )?             # ? takes care of integers of the form a
                       |\.\d+         # mantissa of the form .b
                     )
                     ([eE][+-]?\d+)?  # finally, optionally match an exponent
                   )
		 \D*$/$1/x;
    return $line;
}


#########################################
# Erase undesired characters at the 
# end of string
#########################################
sub trim 
{
    my @out = @_;
    for (@out) 
    {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}
